from graphviz import Digraph, nohtml
from IPython.display import display


class Node:
        
    # if root node then previousNode will be None
    def __init__(self,identifier,values):
        assert all(isinstance(item, int) for item in values),"values has to be a int-list" 
        self.previousNode = None
        self.identifier = identifier # unique identifier
        self.values = values # e.g. Int-Array
        self.nextNodes = [None] * (len(values)+1) # all children are firstly None

class BTree:
            
        
    def __init__(self, M):
        self.graph = Digraph('btree', 'dot', 
                             node_attr = {'shape': 'record', 
                                          'height': '.05',
                                          'fontsize': '10',
                                          'style': 'filled', 
                                          'fillcolor': '#FFFFFF'}, 
                             graph_attr = {'splines': 'line',
                                           'label':'M = '+str(M)})
        
        # all the node that were added
        self.nodeArray = []
        # if someone insert a node with an id
        #  that was already inserted -> throw an error
        self.identifierArray = []
        

        
    def add_node(self, name, elements):   
        i = 1
        res_str = "<f" + str(i) + "> "

        for x in elements:      
            if not isinstance(x, int):
                print(str(x) + ' should be an integer.')
                     
            i = i + 1
            append_str = "|" + str(x) + "|<f" + str(i) + "> "
            res_str = res_str + append_str
        
        self.graph.node(name, nohtml(res_str))
        
        
        if len(self.identifierArray) == 0:
            #root node
            self.nodeArray = [Node(name,elements)]
            self.identifierArray.append(name)
        elif name in self.identifierArray:
            assert False, "Bitte nutze einen anderen Name für diese Node."
        else:
            self.nodeArray.append(Node(name,elements))
            self.identifierArray.append(name)

    
    def add_edge(self, parent, child, n_child):       
        self.graph.edge(parent + ':f' + str(n_child), child)
        
        #search both nodes and add their relations
        parentNode = next((x for x in self.nodeArray if parent == x.identifier), None)
        childNode = next((x for x in self.nodeArray if child == x.identifier), None)
 
        assert parentNode != None and childNode != None, "internal bug: could not find parent or child node in nodeArray."
        
        # add in previous or next node array
        childNode.previousNode = parentNode
        
        isNotInNextArray = next((x for x in parentNode.nextNodes if x != None and childNode.identifier == x.identifier), None) == None
        if isNotInNextArray:
            #index of the node in nextnodes determindes how the tree locks like
            parentNode.nextNodes[n_child-1] = childNode
        
        
        
    def draw(self):
        display(self.graph)
    
    

